@@ -42,7 +42,6 @@
 #endif
 
 GST_DEBUG_CATEGORY (ffmpeg_debug);
-GST_DEBUG_CATEGORY (CAT_PERFORMANCE);
 
 static GMutex gst_avcodec_mutex;
 
@@ -52,7 +51,7 @@ static GMutex gst_avcodec_mutex;
 static inline gboolean
 gst_ffmpeg_avcodec_is_ffmpeg (void)
 {
-  guint av_version = avutil_version ();
+  guint av_version = avcodec_version ();
 
   GST_DEBUG ("Using libavcodec version %d.%d.%d",
       av_version >> 16, (av_version & 0x00ff00) >> 8, av_version & 0xff);
@@ -143,7 +142,6 @@ static gboolean
 plugin_init (GstPlugin * plugin)
 {
   GST_DEBUG_CATEGORY_INIT (ffmpeg_debug, "libav", 0, "libav elements");
-  GST_DEBUG_CATEGORY_GET (CAT_PERFORMANCE, "GST_PERFORMANCE");
 
   /* Bail if not FFmpeg. We can no longer ensure operation with Libav */
   if (!gst_ffmpeg_avcodec_is_ffmpeg ()) {
