@@ -38,6 +38,8 @@
 #include "gstavutils.h"
 #include "gstavviddec.h"
 
+GST_DEBUG_CATEGORY_STATIC (GST_CAT_PERFORMANCE);
+
 #define MAX_TS_MASK 0xff
 
 #define DEFAULT_LOWRES			0
@@ -254,6 +256,8 @@ gst_ffmpegviddec_class_init (GstFFMpegVidDecClass * klass)
   viddec_class->drain = gst_ffmpegviddec_drain;
   viddec_class->decide_allocation = gst_ffmpegviddec_decide_allocation;
   viddec_class->propose_allocation = gst_ffmpegviddec_propose_allocation;
+
+  GST_DEBUG_CATEGORY_GET (GST_CAT_PERFORMANCE, "GST_PERFORMANCE");
 }
 
 static void
@@ -1823,7 +1827,7 @@ gst_ffmpegviddec_handle_frame (GstVideoDecoder * decoder,
       GST_LOG_OBJECT (ffmpegdec, "resized padding buffer to %d",
           ffmpegdec->padded_size);
     }
-    GST_CAT_TRACE_OBJECT (CAT_PERFORMANCE, ffmpegdec,
+    GST_CAT_TRACE_OBJECT (GST_CAT_PERFORMANCE, ffmpegdec,
         "Copy input to add padding");
     memcpy (ffmpegdec->padded, bdata, bsize);
     memset (ffmpegdec->padded + bsize, 0, FF_INPUT_BUFFER_PADDING_SIZE);
@@ -1843,7 +1847,7 @@ gst_ffmpegviddec_handle_frame (GstVideoDecoder * decoder,
 
     if (do_padding) {
       /* add temporary padding */
-      GST_CAT_TRACE_OBJECT (CAT_PERFORMANCE, ffmpegdec,
+      GST_CAT_TRACE_OBJECT (GST_CAT_PERFORMANCE, ffmpegdec,
           "Add temporary input padding");
       memcpy (tmp_padding, data + size, FF_INPUT_BUFFER_PADDING_SIZE);
       memset (data + size, 0, FF_INPUT_BUFFER_PADDING_SIZE);
