@@ -37,6 +37,8 @@ struct _GstFFMpegVidDec
   /* decoding */
   AVCodecContext *context;
   AVFrame *picture;
+  GstVideoMultiviewMode picture_multiview_mode;
+  GstVideoMultiviewFlags picture_multiview_flags;
   gint stride[AV_NUM_DATA_POINTERS];
   gboolean opened;
 
@@ -47,6 +49,11 @@ struct _GstFFMpegVidDec
   gint pic_par_n;
   gint pic_par_d;
   gint pic_interlaced;
+  /* GST_VIDEO_BUFFER_FLAG_RFF | GST_VIDEO_BUFFER_FLAG_TFF */
+  gint pic_field_order;
+  gboolean pic_field_order_changed;
+  GstVideoMultiviewMode cur_multiview_mode;
+  GstVideoMultiviewFlags cur_multiview_flags;
   /* current context */
   gint ctx_ticks;
   gint ctx_time_d;
